/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball, Josh MacDonald
 * Copyright (C) 1997-1998 Jay Painter <jpaint@serv.net><jpaint@gimp.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

#ifndef GTK_DISABLE_DEPRECATED

#ifndef __GTK_UNILIST_H__
#define __GTK_UNILIST_H__

#include <gdk/gdk.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkalignment.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtkhscrollbar.h>
#include <gtk/gtkvscrollbar.h>
#include <gtk/gtkenums.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* unilist flags */
enum {
  GTK_UNILIST_IN_DRAG             = 1 <<  0,
  GTK_UNILIST_ROW_HEIGHT_SET      = 1 <<  1,
  GTK_UNILIST_SHOW_TITLES         = 1 <<  2,
  /* Unused */
  GTK_UNILIST_ADD_MODE            = 1 <<  4,
  GTK_UNILIST_AUTO_SORT           = 1 <<  5,
  GTK_UNILIST_AUTO_RESIZE_BLOCKED = 1 <<  6,
  GTK_UNILIST_REORDERABLE         = 1 <<  7,
  GTK_UNILIST_USE_DRAG_ICONS      = 1 <<  8,
  GTK_UNILIST_DRAW_DRAG_LINE      = 1 <<  9,
  GTK_UNILIST_DRAW_DRAG_RECT      = 1 << 10
}; 

/* cell types */
/*typedef enum
{
  GTK_CELL_EMPTY,
  GTK_CELL_TEXT,
  GTK_CELL_PIXMAP,
  GTK_CELL_PIXTEXT,
  GTK_CELL_WIDGET
} GtkCellType;
*/
typedef enum
{
  GTK_UNILIST_DRAG_NONE,
  GTK_UNILIST_DRAG_BEFORE,
  GTK_UNILIST_DRAG_INTO,
  GTK_UNILIST_DRAG_AFTER
} GtkUniListDragPos;

/*
typedef enum
{
  GTK_BUTTON_IGNORED = 0,
  GTK_BUTTON_SELECTS = 1 << 0,
  GTK_BUTTON_DRAGS   = 1 << 1,
  GTK_BUTTON_EXPANDS = 1 << 2
} GtkButtonAction;
*/
#define GTK_TYPE_UNILIST            (gtk_unilist_get_type ())
#define GTK_UNILIST(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_UNILIST, GtkUniList))
#define GTK_UNILIST_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_UNILIST, GtkUniListClass))
#define GTK_IS_UNILIST(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_UNILIST))
#define GTK_IS_UNILIST_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_UNILIST))
#define GTK_UNILIST_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_UNILIST, GtkUniListClass))


#define GTK_UNILIST_FLAGS(unilist)             (GTK_UNILIST (unilist)->flags)
#define GTK_UNILIST_SET_FLAG(unilist,flag)     (GTK_UNILIST_FLAGS (unilist) |= (GTK_ ## flag))
#define GTK_UNILIST_UNSET_FLAG(unilist,flag)   (GTK_UNILIST_FLAGS (unilist) &= ~(GTK_ ## flag))

#define GTK_UNILIST_IN_DRAG(unilist)           (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_IN_DRAG)
#define GTK_UNILIST_ROW_HEIGHT_SET(unilist)    (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_ROW_HEIGHT_SET)
#define GTK_UNILIST_SHOW_TITLES(unilist)       (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_SHOW_TITLES)
#define GTK_UNILIST_ADD_MODE(unilist)          (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_ADD_MODE)
#define GTK_UNILIST_AUTO_SORT(unilist)         (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_AUTO_SORT)
#define GTK_UNILIST_AUTO_RESIZE_BLOCKED(unilist) (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_AUTO_RESIZE_BLOCKED)
#define GTK_UNILIST_REORDERABLE(unilist)       (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_REORDERABLE)
#define GTK_UNILIST_USE_DRAG_ICONS(unilist)    (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_USE_DRAG_ICONS)
#define GTK_UNILIST_DRAW_DRAG_LINE(unilist)    (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_DRAW_DRAG_LINE)
#define GTK_UNILIST_DRAW_DRAG_RECT(unilist)    (GTK_UNILIST_FLAGS (unilist) & GTK_UNILIST_DRAW_DRAG_RECT)

#define GTK_UNILIST_ROW(_glist_) ((GtkUniListRow *)((_glist_)->data))

/* pointer casting for cells */
#define GTK_CELL_TEXT(cell)     (((GtkCellText *) &(cell)))
#define GTK_CELL_PIXMAP(cell)   (((GtkCellPixmap *) &(cell)))
#define GTK_CELL_PIXTEXT(cell)  (((GtkCellPixText *) &(cell)))
#define GTK_CELL_WIDGET(cell)   (((GtkCellWidget *) &(cell)))

typedef struct _GtkUniList GtkUniList;
typedef struct _GtkUniListClass GtkUniListClass;
typedef struct _GtkUniListColumn GtkUniListColumn;
typedef struct _GtkUniListRow GtkUniListRow;
/*
typedef struct _GtkCell GtkCell;
typedef struct _GtkCellText GtkCellText;
typedef struct _GtkCellPixmap GtkCellPixmap;
typedef struct _GtkCellPixText GtkCellPixText;
typedef struct _GtkCellWidget GtkCellWidget;
*/
typedef gint (*GtkUniListCompareFunc) (GtkUniList     *unilist,
				     gconstpointer ptr1,
				     gconstpointer ptr2);

typedef struct _GtkUniListCellInfo GtkUniListCellInfo;
typedef struct _GtkUniListDestInfo GtkUniListDestInfo;

struct _GtkUniListCellInfo
{
  gint row;
  gint column;
};

struct _GtkUniListDestInfo
{
  GtkUniListCellInfo cell;
  GtkUniListDragPos  insert_pos;
};

struct _GtkUniList
{
  GtkContainer container;
  
  guint16 flags;
  
  /* mem chunks */
  GMemChunk *row_mem_chunk;
  GMemChunk *cell_mem_chunk;

  guint freeze_count;
  
  /* allocation rectangle after the conatiner_border_width
   * and the width of the shadow border */
  GdkRectangle internal_allocation;
  
  /* rows */
  gint rows;
  gint row_height;
  GList *row_list;
  GList *row_list_end;
  
  /* columns */
  gint columns;
  GdkRectangle column_title_area;
  GdkWindow *title_window;
  
  /* dynamicly allocated array of column structures */
  GtkUniListColumn *column;
  
  /* the scrolling window and its height and width to
   * make things a little speedier */
  GdkWindow *unilist_window;
  gint unilist_window_width;
  gint unilist_window_height;
  
  /* offsets for scrolling */
  gint hoffset;
  gint voffset;
  
  /* border shadow style */
  GtkShadowType shadow_type;
  
  /* the list's selection mode (gtkenums.h) */
  GtkSelectionMode selection_mode;
  
  /* list of selected rows */
  GList *selection;
  GList *selection_end;
  
  GList *undo_selection;
  GList *undo_unselection;
  gint undo_anchor;
  
  /* mouse buttons */
  guint8 button_actions[5];

  guint8 drag_button;

  /* dnd */
  GtkUniListCellInfo click_cell;

  /* scroll adjustments */
  GtkAdjustment *hadjustment;
  GtkAdjustment *vadjustment;
  
  /* xor GC for the vertical drag line */
  GdkGC *xor_gc;
  
  /* gc for drawing unselected cells */
  GdkGC *fg_gc;
  GdkGC *bg_gc;
  
  /* cursor used to indicate dragging */
  GdkCursor *cursor_drag;
  
  /* the current x-pixel location of the xor-drag line */
  gint x_drag;
  
  /* focus handling */
  gint focus_row;

  gint focus_header_column;
  
  /* dragging the selection */
  gint anchor;
  GtkStateType anchor_state;
  gint drag_pos;
  gint htimer;
  gint vtimer;
  
  GtkSortType sort_type;
  GtkUniListCompareFunc compare;
  gint sort_column;

  gint drag_highlight_row;
  GtkUniListDragPos drag_highlight_pos;
};

struct _GtkUniListClass
{
  GtkContainerClass parent_class;
  
  void  (*set_scroll_adjustments) (GtkUniList       *unilist,
				   GtkAdjustment  *hadjustment,
				   GtkAdjustment  *vadjustment);
  gint   (*selection_changed)   (GtkUniList       *unilist);
  void   (*refresh)             (GtkUniList       *unilist);
  void   (*select_row)          (GtkUniList       *unilist,
				 gint            row,
				 gint            column,
				 GdkEvent       *event);
  void   (*unselect_row)        (GtkUniList       *unilist,
				 gint            row,
				 gint            column,
				 GdkEvent       *event);
  void   (*row_move)            (GtkUniList       *unilist,
				 gint            source_row,
				 gint            dest_row);
  void   (*click_column)        (GtkUniList       *unilist,
				 gint            column);
  void   (*resize_column)       (GtkUniList       *unilist,
				 gint            column,
                                 gint            width);
  void   (*toggle_focus_row)    (GtkUniList       *unilist);
  void   (*select_all)          (GtkUniList       *unilist);
  void   (*unselect_all)        (GtkUniList       *unilist);
  void   (*undo_selection)      (GtkUniList       *unilist);
  void   (*start_selection)     (GtkUniList       *unilist);
  void   (*end_selection)       (GtkUniList       *unilist);
  void   (*extend_selection)    (GtkUniList       *unilist,
				 GtkScrollType   scroll_type,
				 gfloat          position,
				 gboolean        auto_start_selection);
  void   (*scroll_horizontal)   (GtkUniList       *unilist,
				 GtkScrollType   scroll_type,
				 gfloat          position);
  void   (*scroll_vertical)     (GtkUniList       *unilist,
				 GtkScrollType   scroll_type,
				 gfloat          position);
  void   (*toggle_add_mode)     (GtkUniList       *unilist);
  void   (*abort_column_resize) (GtkUniList       *unilist);
  void   (*resync_selection)    (GtkUniList       *unilist,
				 GdkEvent       *event);
  GList* (*selection_find)      (GtkUniList       *unilist,
				 gint            row_number,
				 GList          *row_list_element);
  void   (*draw_row)            (GtkUniList       *unilist,
				 GdkRectangle   *area,
				 gint            row,
				 GtkUniListRow    *unilist_row);
  void   (*draw_drag_highlight) (GtkUniList        *unilist,
				 GtkUniListRow     *target_row,
				 gint             target_row_number,
				 GtkUniListDragPos  drag_pos);
  void   (*clear)               (GtkUniList       *unilist);
  void   (*fake_unselect_all)   (GtkUniList       *unilist,
				 gint            row);
  void   (*sort_list)           (GtkUniList       *unilist);
  gint   (*insert_row)          (GtkUniList       *unilist,
				 gint            row,
				 gchar          *text[]);
  void   (*remove_row)          (GtkUniList       *unilist,
				 gint            row);
  void   (*set_cell_contents)   (GtkUniList       *unilist,
				 GtkUniListRow    *unilist_row,
				 gint            column,
				 GtkCellType     type,
				 const gchar    *text,
				 guint8          spacing,
				 GdkPixmap      *pixmap,
				 GdkBitmap      *mask);
  void   (*cell_size_request)   (GtkUniList       *unilist,
				 GtkUniListRow    *unilist_row,
				 gint            column,
				 GtkRequisition *requisition);

};

struct _GtkUniListColumn
{
  gchar *title;
  GdkRectangle area;
  
  GtkWidget *button;
  GdkWindow *window;
  
  gint width;
  gint min_width;
  gint max_width;
  GtkJustification justification;
  
  guint visible        : 1;  
  guint width_set      : 1;
  guint resizeable     : 1;
  guint auto_resize    : 1;
  guint button_passive : 1;
};

struct _GtkUniListRow
{
  GtkCell *cell;
  GtkStateType state;
  
  GdkColor foreground;
  GdkColor background;
  
  GtkStyle *style;

  gpointer data;
  GtkDestroyNotify destroy;
  
  guint fg_set     : 1;
  guint bg_set     : 1;
  guint selectable : 1;
};

/* Cell Structures */
/*
struct _GtkCellText
{
  GtkCellType type;
  
  gint16 vertical;
  gint16 horizontal;
  
  GtkStyle *style;

  gchar *text;
};

struct _GtkCellPixmap
{
  GtkCellType type;
  
  gint16 vertical;
  gint16 horizontal;
  
  GtkStyle *style;

  GdkPixmap *pixmap;
  GdkBitmap *mask;
};

struct _GtkCellPixText
{
  GtkCellType type;
  
  gint16 vertical;
  gint16 horizontal;
  
  GtkStyle *style;

  gchar *text;
  guint8 spacing;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
};

struct _GtkCellWidget
{
  GtkCellType type;
  
  gint16 vertical;
  gint16 horizontal;
  
  GtkStyle *style;

  GtkWidget *widget;
};

struct _GtkCell
{
  GtkCellType type;
  
  gint16 vertical;
  gint16 horizontal;
  
  GtkStyle *style;

  union {
    gchar *text;
    
    struct {
      GdkPixmap *pixmap;
      GdkBitmap *mask;
    } pm;
    
    struct {
      gchar *text;
      guint8 spacing;
      GdkPixmap *pixmap;
      GdkBitmap *mask;
    } pt;
    
    GtkWidget *widget;
  } u;
};
*/
GtkType gtk_unilist_get_type (void) G_GNUC_CONST;


/* create a new GtkUniList */
GtkWidget* gtk_unilist_new             (gint   columns);
GtkWidget* gtk_unilist_new_with_titles (gint   columns,
				      gchar *titles[]);

/* set adjustments of unilist */
void gtk_unilist_set_hadjustment (GtkUniList      *unilist,
				GtkAdjustment *adjustment);
void gtk_unilist_set_vadjustment (GtkUniList      *unilist,
				GtkAdjustment *adjustment);

/* get adjustments of unilist */
GtkAdjustment* gtk_unilist_get_hadjustment (GtkUniList *unilist);
GtkAdjustment* gtk_unilist_get_vadjustment (GtkUniList *unilist);

/* set the border style of the unilist */
void gtk_unilist_set_shadow_type (GtkUniList      *unilist,
				GtkShadowType  type);

/* set the unilist's selection mode */
void gtk_unilist_set_selection_mode (GtkUniList         *unilist,
				   GtkSelectionMode  mode);

/* enable unilists reorder ability */
void gtk_unilist_set_reorderable (GtkUniList *unilist,
				gboolean  reorderable);
void gtk_unilist_set_use_drag_icons (GtkUniList *unilist,
				   gboolean  use_icons);
void gtk_unilist_set_button_actions (GtkUniList *unilist,
				   guint     button,
				   guint8    button_actions);

/* freeze all visual updates of the list, and then thaw the list after
 * you have made a number of changes and the updates wil occure in a
 * more efficent mannor than if you made them on a unfrozen list
 */
void gtk_unilist_freeze (GtkUniList *unilist);
void gtk_unilist_thaw   (GtkUniList *unilist);

/* show and hide the column title buttons */
void gtk_unilist_column_titles_show (GtkUniList *unilist);
void gtk_unilist_column_titles_hide (GtkUniList *unilist);

/* set the column title to be a active title (responds to button presses, 
 * prelights, and grabs keyboard focus), or passive where it acts as just
 * a title
 */
void gtk_unilist_column_title_active   (GtkUniList *unilist,
				      gint      column);
void gtk_unilist_column_title_passive  (GtkUniList *unilist,
				      gint      column);
void gtk_unilist_column_titles_active  (GtkUniList *unilist);
void gtk_unilist_column_titles_passive (GtkUniList *unilist);

/* set the title in the column title button */
void gtk_unilist_set_column_title (GtkUniList    *unilist,
				 gint         column,
				 const gchar *title);

/* returns the title of column. Returns NULL if title is not set */
gchar * gtk_unilist_get_column_title (GtkUniList *unilist,
				    gint      column);

/* set a widget instead of a title for the column title button */
void gtk_unilist_set_column_widget (GtkUniList  *unilist,
				  gint       column,
				  GtkWidget *widget);

/* returns the column widget */
GtkWidget * gtk_unilist_get_column_widget (GtkUniList *unilist,
					 gint      column);

/* set the justification on a column */
void gtk_unilist_set_column_justification (GtkUniList         *unilist,
					 gint              column,
					 GtkJustification  justification);

/* set visibility of a column */
void gtk_unilist_set_column_visibility (GtkUniList *unilist,
				      gint      column,
				      gboolean  visible);

/* enable/disable column resize operations by mouse */
void gtk_unilist_set_column_resizeable (GtkUniList *unilist,
				      gint      column,
				      gboolean  resizeable);

/* resize column automatically to its optimal width */
void gtk_unilist_set_column_auto_resize (GtkUniList *unilist,
				       gint      column,
				       gboolean  auto_resize);

gint gtk_unilist_columns_autosize (GtkUniList *unilist);

/* return the optimal column width, i.e. maximum of all cell widths */
gint gtk_unilist_optimal_column_width (GtkUniList *unilist,
				     gint      column);

/* set the pixel width of a column; this is a necessary step in
 * creating a CList because otherwise the column width is chozen from
 * the width of the column title, which will never be right
 */
void gtk_unilist_set_column_width (GtkUniList *unilist,
				 gint      column,
				 gint      width);

/* set column minimum/maximum width. min/max_width < 0 => no restriction */
void gtk_unilist_set_column_min_width (GtkUniList *unilist,
				     gint      column,
				     gint      min_width);
void gtk_unilist_set_column_max_width (GtkUniList *unilist,
				     gint      column,
				     gint      max_width);

/* change the height of the rows, the default (height=0) is
 * the hight of the current font.
 */
void gtk_unilist_set_row_height (GtkUniList *unilist,
			       guint     height);

/* scroll the viewing area of the list to the given column and row;
 * row_align and col_align are between 0-1 representing the location the
 * row should appear on the screnn, 0.0 being top or left, 1.0 being
 * bottom or right; if row or column is -1 then then there is no change
 */
void gtk_unilist_moveto (GtkUniList *unilist,
		       gint      row,
		       gint      column,
		       gfloat    row_align,
		       gfloat    col_align);

/* returns whether the row is visible */
GtkVisibility gtk_unilist_row_is_visible (GtkUniList *unilist,
					gint      row);

/* returns the cell type */
GtkCellType gtk_unilist_get_cell_type (GtkUniList *unilist,
				     gint      row,
				     gint      column);

/* sets a given cell's text, replacing its current contents */
void gtk_unilist_set_text (GtkUniList    *unilist,
			 gint         row,
			 gint         column,
			 const gchar *text);

/* for the "get" functions, any of the return pointer can be
 * NULL if you are not interested
 */
gint gtk_unilist_get_text (GtkUniList  *unilist,
			 gint       row,
			 gint       column,
			 gchar    **text);

/* sets a given cell's pixmap, replacing its current contents */
void gtk_unilist_set_pixmap (GtkUniList  *unilist,
			   gint       row,
			   gint       column,
			   GdkPixmap *pixmap,
			   GdkBitmap *mask);

gint gtk_unilist_get_pixmap (GtkUniList   *unilist,
			   gint        row,
			   gint        column,
			   GdkPixmap **pixmap,
			   GdkBitmap **mask);

/* sets a given cell's pixmap and text, replacing its current contents */
void gtk_unilist_set_pixtext (GtkUniList    *unilist,
			    gint         row,
			    gint         column,
			    const gchar *text,
			    guint8       spacing,
			    GdkPixmap   *pixmap,
			    GdkBitmap   *mask);

gint gtk_unilist_get_pixtext (GtkUniList   *unilist,
			    gint        row,
			    gint        column,
			    gchar     **text,
			    guint8     *spacing,
			    GdkPixmap **pixmap,
			    GdkBitmap **mask);

/* sets the foreground color of a row, the color must already
 * be allocated
 */
void gtk_unilist_set_foreground (GtkUniList *unilist,
			       gint      row,
			       GdkColor *color);

/* sets the background color of a row, the color must already
 * be allocated
 */
void gtk_unilist_set_background (GtkUniList *unilist,
			       gint      row,
			       GdkColor *color);

/* set / get cell styles */
void gtk_unilist_set_cell_style (GtkUniList *unilist,
			       gint      row,
			       gint      column,
			       GtkStyle *style);

GtkStyle *gtk_unilist_get_cell_style (GtkUniList *unilist,
				    gint      row,
				    gint      column);

void gtk_unilist_set_row_style (GtkUniList *unilist,
			      gint      row,
			      GtkStyle *style);

GtkStyle *gtk_unilist_get_row_style (GtkUniList *unilist,
				   gint      row);

/* this sets a horizontal and vertical shift for drawing
 * the contents of a cell; it can be positive or negitive;
 * this is particulary useful for indenting items in a column
 */
void gtk_unilist_set_shift (GtkUniList *unilist,
			  gint      row,
			  gint      column,
			  gint      vertical,
			  gint      horizontal);

/* set/get selectable flag of a single row */
void gtk_unilist_set_selectable (GtkUniList *unilist,
			       gint      row,
			       gboolean  selectable);
gboolean gtk_unilist_get_selectable (GtkUniList *unilist,
				   gint      row);
gboolean gtk_unilist_row_is_selected (GtkUniList *unilist,
				   gint      row);
void gtk_unilist_toggle_row (GtkUniList *unilist,
		 gint      row);

/* prepend/append returns the index of the row you just added,
 * making it easier to append and modify a row
 */
gint gtk_unilist_prepend (GtkUniList    *unilist,
		        gchar       *text[]);
gint gtk_unilist_append  (GtkUniList    *unilist,
			gchar       *text[]);

/* inserts a row at index row and returns the row where it was
 * actually inserted (may be different from "row" in auto_sort mode)
 */
gint gtk_unilist_insert (GtkUniList    *unilist,
		       gint         row,
		       gchar       *text[]);

/* removes row at index row */
void gtk_unilist_remove (GtkUniList *unilist,
		       gint      row);

/* sets a arbitrary data pointer for a given row */
void gtk_unilist_set_row_data (GtkUniList *unilist,
			     gint      row,
			     gpointer  data);

/* sets a data pointer for a given row with destroy notification */
void gtk_unilist_set_row_data_full (GtkUniList         *unilist,
			          gint              row,
			          gpointer          data,
				  GtkDestroyNotify  destroy);

/* returns the data set for a row */
gpointer gtk_unilist_get_row_data (GtkUniList *unilist,
				 gint      row);

/* givin a data pointer, find the first (and hopefully only!)
 * row that points to that data, or -1 if none do
 */
gint gtk_unilist_find_row_from_data (GtkUniList *unilist,
				   gpointer  data);

/* force selection of a row */
void gtk_unilist_select_row (GtkUniList *unilist,
			   gint      row,
			   gint      column);
void
gtk_unilist_real_select_row (GtkUniList *unilist,
		 gint      row);


/* force unselection of a row */
void gtk_unilist_unselect_row (GtkUniList *unilist,
			     gint      row,
			     gint      column);
void
gtk_unilist_real_unselect_row (GtkUniList *unilist,
		 gint      row);

/* undo the last select/unselect operation */
void gtk_unilist_undo_selection (GtkUniList *unilist);

/* clear the entire list -- this is much faster than removing
 * each item with gtk_unilist_remove
 */
void gtk_unilist_clear (GtkUniList *unilist);

/* return the row column corresponding to the x and y coordinates,
 * the returned values are only valid if the x and y coordinates
 * are respectively to a window == unilist->unilist_window
 */
gint gtk_unilist_get_selection_info (GtkUniList *unilist,
			     	   gint      x,
			     	   gint      y,
			     	   gint     *row,
			     	   gint     *column);

/* in multiple or extended mode, select all rows */
void gtk_unilist_select_all (GtkUniList *unilist);

/* in all modes except browse mode, deselect all rows */
void gtk_unilist_unselect_all (GtkUniList *unilist);

/* swap the position of two rows */
void gtk_unilist_swap_rows (GtkUniList *unilist,
			  gint      row1,
			  gint      row2);

/* move row from source_row position to dest_row position */
void gtk_unilist_row_move (GtkUniList *unilist,
			 gint      source_row,
			 gint      dest_row);

/* sets a compare function different to the default */
void gtk_unilist_set_compare_func (GtkUniList            *unilist,
				 GtkUniListCompareFunc  cmp_func);

/* the column to sort by */
void gtk_unilist_set_sort_column (GtkUniList *unilist,
				gint      column);

/* how to sort : ascending or descending */
void gtk_unilist_set_sort_type (GtkUniList    *unilist,
			      GtkSortType  sort_type);

/* sort the list with the current compare function */
void gtk_unilist_sort (GtkUniList *unilist);

/* Automatically sort upon insertion */
void gtk_unilist_set_auto_sort (GtkUniList *unilist,
			      gboolean  auto_sort);

/* Private function for unilist, ctree */

PangoLayout *_gtk_unilist_create_cell_layout (GtkUniList       *unilist,
					    GtkUniListRow    *unilist_row,
					    gint            column);

#ifdef __cplusplus
}
#endif				/* __cplusplus */

#endif				/* __GTK_UNILIST_H__ */

#endif /* GTK_DISABLE_DEPRECATED */
